@@ -93,6 +93,7 @@
             code:port-bind?
             code:port-name
             code:port-release
+            code:port-reply
             code:port-type
             code:pump?
             code:reply
@@ -332,7 +333,12 @@
                                (is? <location>)))
               (disjoin (is? <blocking>) (is? <blocking-compound>))
               (parent o <model>))) '()
-      o))
+              o))
+
+(define-method (code:port-reply o)
+  (let ((port (.port o)))
+    (if (ast:provides? port) port
+        '())))
 
 
 ;;;
